{***************************************************************************************************
*                                 Copyright  2004, Sergej Malik                                   *
*                                                                                                  *
*        OscKnob.pas -                *
*                                                                                                  *
*          : 16.11.04                                                                        *
***************************************************************************************************}
unit OscKnob;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, StdCtrls, ExtCtrls, Math;

{***************************************************************************************************
*TODO: 0.0                       ,                                      *
***************************************************************************************************}

type
  TROscKnobData = record                //   
    dValueReal  : double;               //   
    dAngle      : double;               //     
    dValue      : double;               //     
    dExp        : double;               //   
    strDimension: string;               //  
    strHint     : string;               //   
  end;

  TOscKnobData = array of TROscKnobData;//    

{***************************************************************************************************
*TODO: 0.1                         -                                     *
***************************************************************************************************}
procedure Register;             //    TOscKnob

procedure fill_line_gradient(x0, y0, x1, y1, len, cl1, cl2: integer; btFill: TBitMap);              //   

procedure fill_circle_line_gradient(r, x, y, cl1, cl2: integer; angle:double; btFill: TBitMap);     //    

procedure paint_knob_circles(r1, r2, x, y, cl1_r1, cl2_r1, cl1_r2, cl2_r2: integer;                 //   
                             angle1, angle2:double; btFill: TBitMap);

procedure paint_circle_scale_label(r_in, r_out, x, y, cl: integer; angle:double; btSacle: TBitMap); //   

function del_all_space(str:string):string;                                                          //     

procedure paint_circle_scale(r_in, r_out, r_out_sm, x, y, cl, cl_sm, font_size, index_max: integer; //   
                             angle_start, angle_stop: double; slScaleCode: TStrings;
                             btSacle: TBitMap; var ScaleData: TOscKnobData; FontName:string='Tahoma');

procedure paint_cursor(r, h, x, y, cl: integer; angle, angle_cursor: double; cnCursor: TCanvas);    //  


{***************************************************************************************************
*TODO: 0.2                                           *
***************************************************************************************************}

{***************************************************************************************************
*TODO: 0.3                       : TOscKnobLabel                                    *
***************************************************************************************************}
type
  TOscKnobLabel = class(TCustomLabel)
  private
    { Private declarations }
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override; //    

  public
    constructor Create(AOwner: TComponent); override;

  published
    property BiDiMode;
    property Caption;
    property Color;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Font;
    property Height;
    property Left;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Top;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property Width;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;


{***************************************************************************************************
*TODO: 0.4                           : TOscKnob                                     *
***************************************************************************************************}
  TOscKnob = class(TCustomControl)
  private
    //-----------------------------------------  -------------------------------------------
    FX: integer;                //      
    FY: integer;

    FRCirDown: integer;         //   () 
    FRCirUp: integer;           //   () 
    FRScaleIn: integer;         //  
    FRScaleOut: integer;        //    
    FRScaleOutSm: integer;      //    
    FRCursorIn: integer;        //  ( )
    FHCursor: integer;          //   : r+h -   ( )

    FColor: TColor;             //  
    FColorSelect: TColor;       //   
    FColor1CirDown: TColor;     //     
    FColor2CirDown: TColor;     //      
    FColor1CirUp: TColor;       //     
    FColor2CirUp: TColor;       //      
    FColorScale: TColor;        //   
    FColorScaleSm: TColor;      //   
    FColorCursor: TColor;       //  
    FColorCursorMove: TColor;   //     

    FAngle: double;             //    (0 - 12:00,    ), 
    FAngleCursor: double;       //    , 
    FAngleFillCirDown: double;  //      (0 - 12:00,    ), 
    FAngleFillCirUp: double;    //     (0 - 12:00,    ), 
    FAngleStartScale: double;   //       >0 (0 - 12:00,    ), 
    FAngleStopScale: double;    //    <0 (0 - 12:00,    ), 

    FCursorMove: boolean;       //   
    FMouseOn: boolean;          //  ,      
    FCaptionDimHide: boolean;   //    
    FCaptionAvtoCenter: boolean;//    
    FHintAvtoSet: boolean;      //    

    FIndexSelectData: integer;  //    
    FIndexMaxEnable: integer;   //   
                
    FFont: TFont;               //  
    FCaption: TOscKnobLabel;    //  
    FBgBitMap: TBitMap;         // BitMap 
    FScaleCode: TStrings;       //   
    FOscKnobData: TOscKnobData; //   
    FComboBoxSelect: TComboBox; //  

    //------------------------------------------  -------------------------------------------
    FOnChange: TNotifyEvent;    //  

    //----------------------------     -----------------------------------
    procedure SetAngleStartScale(const Value: double);
    procedure SetAngleStopScale(const Value: double);
    procedure SetAngle(const Value: double);
    procedure SetAngleCursor(const Value: double);
    procedure SetAngleFillCirDown(const Value: double);
    procedure SetAngleFillCirUp(const Value: double);
    procedure SetColor(const Value: TColor);
    procedure SetColor1CirDown(const Value: TColor);
    procedure SetColor1CirUp(const Value: TColor);
    procedure SetColor2CirDown(const Value: TColor);
    procedure SetColor2CirUp(const Value: TColor);
    procedure SetColorCursor(const Value: TColor);
    procedure SetColorCursorMove(const Value: TColor);
    procedure SetColorScale(const Value: TColor);
    procedure SetColorScaleSm(const Value: TColor);
    procedure SetColorSelect(const Value: TColor);
    procedure SetFFont(const Value: TFont);
    procedure SetHCursor(const Value: integer);
    procedure SetRCirDown(const Value: integer);
    procedure SetRCirUp(const Value: integer);
    procedure SetRCursorIn(const Value: integer);
    procedure SetRScaleIn(const Value: integer);
    procedure SetRScaleOut(const Value: integer);
    procedure SetRScaleOutSm(const Value: integer);
    procedure SetScaleCode(const Value: TStrings);
    procedure SetX(const Value: integer);
    procedure SetY(const Value: integer);
    procedure SetCaptionDimHide(const Value: boolean);
    procedure SetCaptionAvtoCenter(const Value: boolean);
    procedure SetHintAvtoSet(const Value: boolean);
    procedure SetComboBoxSelect(const Value: TComboBox);
    procedure SetIndexMaxEnable(const Value: integer);
    procedure SetIndexSelectData(const Value: integer);
    function GetCurAngle: double;
    function GetCurDimension: string;
    function GetCurExp: double;
    function GetCurHint: string;
    function GetCurValue: double;
    function GetCurValueReal: double;

  protected
    procedure paint_background;                         //  
    procedure Paint; override;                          //  
    procedure set_exist_angle(angle: double);           //   
    procedure set_cursor_pos(x, y: integer);            //  
    procedure change_index;                             //    
    procedure change_by_combobox(Sender: TObject);      //    ComboBox

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;        //     

    procedure CMMouseEnter(var msg: TMessage); message CM_MOUSEENTER;                       //     
    procedure CMMouseLeave(var msg: TMessage); message CM_MOUSELEAVE;                       //     

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override; //    
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;                       //   
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;   //    

    procedure WMSetFocus(var msg: TWMSETFOCUS); message WM_SETFOCUS;                        //   
    procedure WMKillFocus(var msg: TWMKILLFOCUS); message WM_KILLFOCUS;                     //   

    procedure KeyDown(var Key: Word; Shift: TShiftState); override;                         //      
    procedure WMGetDlgCode(var msg: TWMGetDlgCode); message WM_GETDLGCODE;                  //    
    procedure MSHMouseWheel(var msg: TMSHMouseWheel); message WM_MOUSEWHEEL;                //    

    procedure CMEnabledChanged(var msg: TMessage); message cm_EnabledChanged;               //    Enable
    procedure CMVisibleChanged(var msg: TMessage); message cm_VisibleChanged;               //    Visible

    procedure WMSize(var msg: TWMSize); message WM_SIZE;                                    //   

  public
    constructor Create(AOwner: TComponent); override;   // 
    destructor Destroy; override;                       // 

    procedure SetCursorByValueReal(Value: double);      //     ValueReal

  published
    //-----------------------------------------  -------------------------------------------
    property ParentShowHint;                            //   
    property ShowHint;
    property TabStop default True;
    property TabOrder;
    property PopupMenu;
    property Visible;
    property Enabled;

    property X: integer read FX write SetX default 45;  //      
    property Y: integer read FY write SetY default 35;

    property RCirDown: integer    read FRCirDown    write SetRCirDown    default 20;    //   () 
    property RCirUp:   integer    read FRCirUp      write SetRCirUp      default 12;    //   () 
    property RScaleIn: integer    read FRScaleIn    write SetRScaleIn    default 20;    //  
    property RScaleOut: integer   read FRScaleOut   write SetRScaleOut   default 26;    //    
    property RScaleOutSm: integer read FRScaleOutSm write SetRScaleOutSm default 24;    //    
    property RCursorIn: integer   read FRCursorIn   write SetRCursorIn   default 13;    //  ( )
    property HCursor: integer     read FHCursor     write SetHCursor     default 4;     //   : r+h -   ( )

    property IndexMaxEnable: integer read FIndexMaxEnable  write SetIndexMaxEnable  default -1;         //   

    property Color: TColor           read FColor           write SetColor           default clBtnFace;  //  
    property ColorSelect: TColor     read FColorSelect     write SetColorSelect     default $E8E8E8;    //   
    property Color1CirDown: TColor   read FColor1CirDown   write SetColor1CirDown   default $EBEBEB;    //     
    property Color2CirDown: TColor   read FColor2CirDown   write SetColor2CirDown   default $848484;    //      
    property Color1CirUp: TColor     read FColor1CirUp     write SetColor1CirUp     default clBtnFace;  //     
    property Color2CirUp: TColor     read FColor2CirUp     write SetColor2CirUp     default clBtnFace;  //      
    property ColorScale: TColor      read FColorScale      write SetColorScale      default clBlack;    //   
    property ColorScaleSm: TColor    read FColorScaleSm    write SetColorScaleSm    default clBtnShadow;//   
    property ColorCursor: TColor     read FColorCursor     write SetColorCursor     default clRed;      //  
    property ColorCursorMove: TColor read FColorCursorMove write SetColorCursorMove default clRed;      //     

    property AngleStartScale: double  read FAngleStartScale  write SetAngleStartScale;  //       >0 (0 - 12:00,    ),  ( SetAngle)
    property AngleStopScale: double   read FAngleStopScale   write SetAngleStopScale;   //    <0 (0 - 12:00,    ),  ( SetAngle)
    property Angle: double            read FAngle            write SetAngle;            //    (0 - 12:00,    ), 

    property AngleCursor: double      read FAngleCursor      write SetAngleCursor;      //    , 
    property AngleFillCirDown: double read FAngleFillCirDown write SetAngleFillCirDown; //      (0 - 12:00,    ), 
    property AngleFillCirUp: double   read FAngleFillCirUp   write SetAngleFillCirUp;   //     (0 - 12:00,    ), 

    property CaptionDimHide: boolean    read FCaptionDimHide    write SetCaptionDimHide    default false;   //    
    property CaptionAvtoCenter: boolean read FCaptionAvtoCenter write SetCaptionAvtoCenter default true;    //    
    property HintAvtoSet: boolean       read FHintAvtoSet       write SetHintAvtoSet       default true;    //    

    property Font: TFont               read FFont           write SetFFont;             //  
    property ScaleCode: TStrings       read FScaleCode      write SetScaleCode;         //   
    property Caption: TOscKnobLabel    read FCaption;                                   //  
    property ComboBoxSelect: TComboBox read FComboBoxSelect write SetComboBoxSelect;    //  

    property IndexSelectData : integer      read FIndexSelectData write SetIndexSelectData; //    
    property OscKnobData     : TOscKnobData read FOscKnobData;                              //   

    property CurValueReal    : double       read GetCurValueReal;                           //   
    property CurAngle        : double       read GetCurAngle;                               //     
    property CurValue        : double       read GetCurValue;                               //     
    property CurExp          : double       read GetCurExp;                                 //   
    property CurDimension    : string       read GetCurDimension;                           //  
    property CurHint         : string       read GetCurHint;                                //   

    //------------------------------------------  -------------------------------------------
    property OnClick;                                   //   
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;     //      

  end;

{***************************************************************************************************
*TODO: 0.4                       : TOscKnobStrings                                  *
***************************************************************************************************}
  TOscKnobStrings = class(TStringList)                      //  
  private
    OscKnob: TOscKnob;                                      // 
  protected
    procedure SetUpdateState(Updating: Boolean); override;  //    
  end;


{***************************************************************************************************
*TODO: 0.x                                                                            *
***************************************************************************************************}

implementation

{***************************************************************************************************
*TODO: 1.0                                                                              *
***************************************************************************************************}

//TODO: 1.01 ----------------------    ------------------------------------
// x0, y0 -   
// x1, y1 -   
// len -  
// cl1 -   
// cl2 -   
// btFill - BitMap  
procedure fill_line_gradient(x0, y0, x1, y1, len, cl1, cl2: integer; btFill: TBitMap);
var
  i, dx, dy: integer;
  cr, cg, cb, cr1, cg1, cb1, cr2, cg2, cb2: byte;
  h, len2, dxh, dyh, k, b, dlx, dly, x, y, lx0, ly0, lx1, ly1: double;

  function get_gradient_color:TColor;                                   //    
  begin
    cr:=Round(cr1-(cr1-cr2)/h*i);                                       //    
    cg:=Round(cg1-(cg1-cg2)/h*i);
    cb:=Round(cb1-(cb1-cb2)/h*i);
    get_gradient_color:=RGB(cr,cg,cb);
  end;

begin
  PatBlt(btFill.Canvas.Handle,0,0,btFill.Width,btFill.Height,WHITENESS);//  

  cr1:=GetRValue(cl1); cg1:=GetGValue(cl1); cb1:=GetBValue(cl1);        //   1
  cr2:=GetRValue(cl2); cg2:=GetGValue(cl2); cb2:=GetBValue(cl2);        //   2

  dx:=x1-x0;                                                            //   
  dy:=y1-y0;                                                            //   
  h:=sqrt(sqr(dx)+sqr(dy));                                             //   
  if h=0 then exit;                                                     //     
  dxh:=dx/h;                                                            //  dx  h
  dyh:=dy/h;                                                            //  dy  h
  len2:=len/2;                                                          //    

  if dx=0 then                                                          //    
    for i:=0 to Round(h-1) do                                           //   
      with btFill.Canvas do
      begin
        Pen.Color:=get_gradient_color;                                  //     i
        MoveTo(Round(x0-len2),Round(y0+dyh*i));                         //     x0  len2
        LineTo(Round(x0+len2),Round(y0+dyh*i));                         //    x0  len2
      end
  else if dy=0 then begin                                               //    
    for i:=0 to Round(h-1) do                                           //   
      with btFill.Canvas do
      begin
        Pen.Color:=get_gradient_color;                                  //     i
        MoveTo(Round(x0+dxh*i),Round(y0-len2));                         //    y0  len2
        LineTo(Round(x0+dxh*i),Round(y0+len2));                         //    y0  len2
      end
  end
  else begin                                                            //    
    k:=(y1-y0)/(x1-x0);                                                 //  
    b:=y0-x0*k;
    dlx:=len2*dy/h;                                                     // len2   
    dly:=len2*dx/h;
    for i:=0 to Round(h-1) do                                           //   
      with btFill.Canvas do
      begin
        Pen.Color:=get_gradient_color;                                  //     i
        y:=y0+dyh*i;                                                    //     h
        x:=(y-b)/k;                                                     //  x   h
        lx0:=x-dlx; ly0:=y+dly;                                         //   -     
        lx1:=x+dlx; ly1:=y-dly;                                         //   -     
        MoveTo(Round(lx0    ), Round(ly0    )); LineTo(Round(lx1    ), Round(ly1    ));
        MoveTo(Round(lx0+0.5), Round(ly0+0.5)); LineTo(Round(lx1+0.5), Round(ly1+0.5));
        MoveTo(Round(lx0+0.5), Round(ly0-0.5)); LineTo(Round(lx1+0.5), Round(ly1-0.5));
        MoveTo(Round(lx0-0.5), Round(ly0+0.5)); LineTo(Round(lx1-0.5), Round(ly1+0.5));
        MoveTo(Round(lx0-0.5), Round(ly0-0.5)); LineTo(Round(lx1-0.5), Round(ly1-0.5));
      end;
  end;
end;


//TODO: 1.02 --------------     ---------------------------------
// r -  
// x,y -     btFill
// cl1 -   
// cl2 -   
// angle -   (0 - 12:00,    ), 
// btFill - BitMap  
procedure fill_circle_line_gradient(r, x, y, cl1, cl2: integer; angle:double; btFill: TBitMap);
var
  dx, dy, d: integer;
  btFLG, btCircle: TBitMap;
begin
  btFLG:=TBitMap.Create;                //      
  btCircle:=TBitMap.Create;             //      

  d:=2*r;                               //   
  btFLG.Width:=d;                       //     
  btFLG.Height:=d;
  btCircle.Width:=d;
  btCircle.Height:=d;

  dx:=Round(r*sin(DegToRad(angle)));    //         
  dy:=Round(r*cos(DegToRad(angle)));

  fill_line_gradient(r-dx,r-dy,r+dx,r+dy,d,cl1,cl2,btFLG);                  //   btFLG

  PatBlt(btCircle.Canvas.Handle,0,0,d,d,BLACKNESS);                         //  

  btCircle.Canvas.Brush.Color:=clWhite;                                     //      
  btCircle.Canvas.Brush.Style:=bsSolid;
  btCircle.Canvas.Pen.Color:=clWhite;
  btCircle.Canvas.Ellipse(0,0,d,d);

  BitBlt(btFLG.Canvas.Handle,0,0,d,d,btCircle.Canvas.Handle,0,0,SRCAND);            //  FLG     
  BitBlt(btCircle.Canvas.Handle,0,0,d,d,btCircle.Canvas.Handle,0,0,DSTINVERT);      //      
  BitBlt(btFill.Canvas.Handle,x-r,y-r,x+r,y+r,btCircle.Canvas.Handle,0,0,SRCAND);   //   btFill   r,x,y
  BitBlt(btFill.Canvas.Handle,x-r,y-r,x+r,y+r,btFLG.Canvas.Handle,0,0,SRCPAINT);    //     

  btCircle.Free;                        //        
  btFLG.Free;                           //        
end;


//TODO: 1.03 -----------------------    --------------------------------
// r1 -   ( )   
// r2 -   ( )
// x,y -     btFill
// cl1_r1 -     
// cl2_r1 -      
// cl1_r2 -     
// cl2_r2 -      
// angle1 -      (0 - 12:00,    ), 
// angle2 -     (0 - 12:00,    ), 
// btFill - BitMap  
procedure paint_knob_circles(r1, r2, x, y, cl1_r1, cl2_r1, cl1_r2, cl2_r2: integer; angle1, angle2:double; btFill: TBitMap);
begin
  fill_circle_line_gradient(r1,x,y,cl1_r1,cl2_r1,angle1,btFill);    //   
  if cl1_r2=cl2_r2 then                                             //     
  begin
    btFill.Canvas.Brush.Style:=bsSolid;
    btFill.Canvas.Brush.Color:=cl1_r2;
    btFill.Canvas.Pen.Color:=cl1_r2;
    btFill.Canvas.Ellipse(x-r2,y-r2,x+r2,y+r2);
  end
  else
    fill_circle_line_gradient(r2,x,y,cl1_r2,cl2_r2,angle2,btFill);
end;


//TODO: 1.04 ---------------------     -----------------------------------------
// r_in  -  
// r_out -  
// x,y   -     btSacle
// cl    -   
// angle   -    (0 - 12:00,    ), 
// btSacle - BitMap   
procedure paint_circle_scale_label(r_in, r_out, x, y, cl: integer; angle:double; btSacle: TBitMap);
begin
  btSacle.Canvas.Pen.Color:=cl;                                                                     //  
  btSacle.Canvas.MoveTo(x-Round(r_in*sin(DegToRad(angle))), y-Round(r_in*cos(DegToRad(angle))));    //    
  btSacle.Canvas.LineTo(x-Round(r_out*sin(DegToRad(angle))),y-Round(r_out*cos(DegToRad(angle))));   //    
end;


//TODO: 1.05 --------------------      ------------------------------------
function del_all_space(str:string):string;
var
  i: integer;
  s: string;
begin
  s:='';
  for i:=1 to Length(str) do if str[i]<>' ' then s:=s+str[i];
  del_all_space:=s;
end;


//TODO: 1.06 -------------------------    ----------------------------------------
// r_in     -  
// r_out    -    
// r_out_sm -    
// x, y     -     btSacle
// cl       -   
// cl_sm    -   
// font_size   -  
// index_max   -   
// angle_start -       >0 (0 - 12:00,    ), 
// angle_stop  -    <0 (0 - 12:00,    ), 
// slScaleCode -   
// btSacle     - BitMap   
// ScaleData   -   
// FontName    -  
procedure paint_circle_scale(r_in, r_out, r_out_sm, x, y, cl, cl_sm, font_size, index_max: integer; angle_start, angle_stop: double;
                             slScaleCode: TStrings; btSacle: TBitMap; var ScaleData: TOscKnobData; FontName:string='Tahoma');
var
  i, cur_r, cur_cl, cur_font_size, len_str, tw, th, offset_x, offset_y: integer;
  value, exp, delta_angle, cur_angle, tw2, tx, ty, tws, ths: double;
  strCode, strDimension, strPrint, strHint: string;
  hide, rotate_text: boolean;
  lfFont: TLogFont;

  function decode_str:boolean;
  var
    c: char;
    strValue: string;
    pos1, pos2, set_font_size: integer;
  begin
    pos1:=pos('#',strCode);                                     //   
    if pos1=0 then                                              //      
    begin
      decode_str:=false;                                        //   ,   
      exit;                                                     //   
    end;
    strCode[pos1]:=' ';                                         //    

    pos2:=pos('#',strCode);                                     //   
    if pos2=0 then                                              //      
    begin
      decode_str:=false;                                        //   ,   
      pos2:=len_str+1;                                          //       
    end
    else                                                        //     
      decode_str:=true;                                         //        

    if pos1=len_str then                                        //   #    
    begin
      decode_str:=false;                                        //   ,   
      exit;                                                     //   
    end;

    c:=AnsiUpperCase(strCode[pos1+1])[1];                       //  
    if c='#' then exit;                                         //   #  

    strValue:='';                                               //      
    if ((pos1+2)<=len_str) and (strCode[pos1+2]='=') then       //    #X=...
      inc(pos1);                                                //     =
    if (pos2-pos1)>2 then                                       //     
      strValue:=Copy(strCode,pos1+2,pos2-pos1-2);               //    
    strValue:=Trim(strValue);                                   //        

    try
      case c of                                                 //    
        'V': value:=StrToFloat(strValue);                       //  
        'E': exp:=StrToFloat(strValue);                         //   
        'S': begin                                              //  
               cur_r:=r_out_sm;
               cur_cl:=cl_sm;
             end;
        'D': strDimension:=strValue;                            //  
        'H': strHint:=strValue;                                 //   
        'P': hide:=true;                                        //  
        'X': offset_x:=StrToInt(strValue);                      //    
        'Y': offset_y:=StrToInt(strValue);                      //    
        'R': rotate_text:=true;                                 //  
        'F': begin                                              //  
                set_font_size:=StrToInt(strValue);              //  (  )
                if (strValue[1]='-') or (strValue[1]='+') then  //   
                  cur_font_size:=cur_font_size+set_font_size
                else                                            //   
                  cur_font_size:=set_font_size;
             end;
      end;
    except
    end;
  end;

begin
  SetLength(ScaleData,slScaleCode.Count);                       //     

  if slScaleCode.Count=0 then exit;                             //   
  SetGraphicsMode(btSacle.Canvas.Handle,GM_ADVANCED);           //    

  delta_angle:=0;                                               //   
  if slScaleCode.Count>1 then
    delta_angle:=(angle_start-angle_stop)/(slScaleCode.Count-1);

  for i:=0 to slScaleCode.Count-1 do                            //   
  begin
    strCode:=slScaleCode.Strings[i];                            //   
    len_str:=Length(strCode);                                   //    

    value:=0;                                                   // #V...    0
    exp:=0;                                                     // #E...     0
    cur_r:=r_out;                                               // #S       
    cur_cl:=cl;                                                 // #S         
    strDimension:='';                                           // #D...     
    strHint:='';                                                // #H...     
    hide:=false;                                                // #P       
    offset_x:=0;                                                // #X...      
    offset_y:=0;                                                // #Y...      
    rotate_text:=false;                                         // #R        
    cur_font_size:=font_size;                                   // #F...    

    while decode_str do ;

    cur_angle:=angle_start - delta_angle*i;                                     //   
    if (index_max<>-1) and (i>=index_max) then cur_cl:=clMedGray;               //       
    paint_circle_scale_label(r_in,cur_r,x,y,cur_cl,cur_angle,btSacle);          //  

    ScaleData[i].dValueReal:=value*power(10,exp);                               //   
    ScaleData[i].dValue:=value;                                                 //     
    ScaleData[i].dExp:=exp;                                                     //     
    ScaleData[i].dAngle:=cur_angle;                                             //   
    ScaleData[i].strDimension:=strDimension;                                    //  
    ScaleData[i].strHint:=strHint;                                              //   

    if not hide then                                                            //    
    begin
      FillChar(lfFont,SizeOf(TLogFont),0);                                      //    
      lfFont.lfHeight:=cur_font_size;                                           //   
      if rotate_text then                                                       //   
      begin
        lfFont.lfEscapement:=Round(cur_angle*10);
        lfFont.lfOrientation:=Round(cur_angle*10);
      end;
      lfFont.lfCharSet:=RUSSIAN_CHARSET;
      lfFont.lfQuality:=PROOF_QUALITY;
      StrPCopy(lfFont.lfFaceName,FontName);
      btSacle.Canvas.Font.Handle:=CreateFontIndirect(lfFont);                   //     

      strPrint:=FloatToStr(value)+strDimension;                                 //    
      tw:=btSacle.Canvas.TextWidth(strPrint);                                   //    
      th:=btSacle.Canvas.TextHeight(strPrint);                                  //    
      tw2:=tw/2;                                                                //   

      if rotate_text then                                                       //   
      begin
        tx:=x-(cur_r+th)*sin(DegToRad(cur_angle))-tw2*cos(DegToRad(cur_angle)); //      
        ty:=y-(cur_r+th)*cos(DegToRad(cur_angle))+tw2*sin(DegToRad(cur_angle));
      end
      else begin
        if cur_angle>=0 then tws:=-1 else tws:=0;                               //       
        if abs(cur_angle)>=90 then ths:=0 else ths:=-1;
        tx:=x-(cur_r-2)*sin(DegToRad(cur_angle))+tws*tw;                        //      
        ty:=y-(cur_r-2)*cos(DegToRad(cur_angle))+ths*th;
      end;

      btSacle.Canvas.Font.Color:=cur_cl;                                        //   
      btSacle.Canvas.Brush.Style:=bsClear;                                      //    
      btSacle.Canvas.TextOut(Round(tx)+offset_x,Round(ty)+offset_y,strPrint);   //  
    end;
  end;
end;


//TODO: 1.07 ----------------------------   ----------------------------------------
// r     -   ( )
// h     -   : r+h -   ( )
// x, y  -     btCursor
// cl    -  
// angle -    (0 - 12:00,    ), 
// angle_cursor -    , 
// cnCursor     - Canvas   
procedure paint_cursor(r, h, x, y, cl: integer; angle, angle_cursor: double; cnCursor: TCanvas);
var
  x1, y1, x2, y2, x3, y3: integer;
  ptCursor: array [0..2] of TPoint;
begin
  cnCursor.Pen.Color:=cl;
  cnCursor.Brush.Color:=cl;
  cnCursor.Brush.Style:=bsSolid;

  x1:=Round(x-  r  *sin(DegToRad(angle+angle_cursor)));
  y1:=Round(y-  r  *cos(DegToRad(angle+angle_cursor)));
  x2:=Round(x-(r+h)*sin(DegToRad(angle)));
  y2:=Round(y-(r+h)*cos(DegToRad(angle)));
  x3:=Round(x-  r  *sin(DegToRad(angle-angle_cursor)));
  y3:=Round(y-  r  *cos(DegToRad(angle-angle_cursor)));

  ptCursor[0].X:=x1; ptCursor[0].Y:=y1;
  ptCursor[1].X:=x2; ptCursor[1].Y:=y2;
  ptCursor[2].X:=x3; ptCursor[2].Y:=y3;

  cnCursor.Polygon(ptCursor);
end;


{***************************************************************************************************
*TODO: 2.0                           : TOscKnob                                     *
***************************************************************************************************}

//TODO: 2.00 -----------------------    TOscKnob -----------------------------
procedure Register;
begin
  RegisterComponents('New', [TOscKnob]);
end;


//TODO: 2.01 ---------------------------------  -----------------------------------------
constructor TOscKnob.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);                     //  

  Width:=90;                                    //   
  Height:=60;

  TabStop:=True;                                //    Tab-
  Cursor:=crHandPoint;                          //      

  FX:=45;                                       //      
  FY:=35;
  FRCirDown:=20;                                //   () 
  FRCirUp:=12;                                  //   () 
  FRScaleIn:=20;                                //  
  FRScaleOut:=26;                               //    
  FRScaleOutSm:=24;                             //    
  FRCursorIn:=13;                               //  ( )
  FHCursor:=4;                                  //   : r+h -   ( )

  FColor:=clBtnFace;                            //  
  FColorSelect:=$E8E8E8;                        //   
  FColor1CirDown:=RGB(235,235,235);             //     
  FColor2CirDown:=RGB(132,132,132);             //      
  FColor1CirUp:=clBtnFace;                      //     
  FColor2CirUp:=clBtnFace;                      //      
  FColorScale:=clBlack;                         //   
  FColorScaleSm:=clBtnShadow;                   //   
  FColorCursor:=clRed;                          //  
  FColorCursorMove:=clRed;                      //     

  FAngle:=130;                                  //    (0 - 12:00,    ), 
  FAngleCursor:=10;                             //    , 
  FAngleFillCirDown:=45;                        //      (0 - 12:00,    ), 
  FAngleFillCirUp:=225;                         //     (0 - 12:00,    ), 
  FAngleStartScale:=130;                        //       >0 (0 - 12:00,    ), 
  FAngleStopScale:=-130;                        //    <0 (0 - 12:00,    ), 

  FCursorMove:=false;                           //   
  FMouseOn:=false;                              //  ,      
  FCaptionDimHide:=false;                       //    
  FCaptionAvtoCenter:=true;                     //    
  FHintAvtoSet:=true;                           //    

  FIndexSelectData:=0;                          //    
  FIndexMaxEnable:=-1;                          //     

  FFont:=TFont.Create;                          //  
  FFont.Size:=12;                               //  
  FFont.Name:='Arial';                          //  

  FBgBitMap:=TBitMap.Create;                    // BitMap 
  FBgBitMap.Width:=90;                          //  BitMap 
  FBgBitMap.Height:=60;

  FCaption:=TOscKnobLabel.Create(Self);         //   
  FCaption.Parent:=self;                        //     
  FCaption.FreeNotification(Self);              //    
  FCaption.Font.Size:=7;                        //   
  FCaption.Transparent:=true;                   //  
  FCaption.AutoSize:=false;                     //         

  FScaleCode:=TOscKnobStrings.Create;           //   
  TOscKnobStrings(FScaleCode).OscKnob := Self;  //   

  FScaleCode.Add('#v=0  #h=set 0  #y=-4 #x=-1');//   
  FScaleCode.Add('#v=1  #h=set 1  #s #p');
  FScaleCode.Add('#v=2  #h=set 2  #y=+4 #x=-1');
  FScaleCode.Add('#v=3  #h=set 3  #s #p');
  FScaleCode.Add('#v=4  #h=set 4  #y=+6 #x=-1');
  FScaleCode.Add('#v=5  #h=set 5  #s #p');
  FScaleCode.Add('#v=6  #h=set 6  #y=+6 #x=+2');
  FScaleCode.Add('#v=7  #h=set 7  #s #p');
  FScaleCode.Add('#v=8  #h=set 8  #y=+4 #x=+3');
  FScaleCode.Add('#v=9  #h=set 9  #s #p');
  FScaleCode.Add('#v=10 #h=set 10 #y=-4 #x=+3');

  paint_background;                             //  

  ControlStyle:=ControlStyle+[csOpaque];        //   
end;

//TODO: 2.02 --------------------------  OscKnobLabel -----------------------------------
constructor TOscKnobLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Name:='OscKnobLabel';
  SetSubComponent(True);
end;


//TODO: 2.03 ---------------------------------  ------------------------------------------
destructor TOscKnob.Destroy;
begin
  FFont.Free;                                   //     
  FBgBitMap.Free;                               //    BitMap 
  FCaption.Free;                                //     
  FScaleCode.Free;                              //      
  SetLength(FOscKnobData,0);                    //    

  inherited;                                    //  
end;


//TODO: 2.04 -------------------------------   ---------------------------------------
procedure TOscKnob.paint_background;
var
  i: integer;
begin
  FBgBitMap.Canvas.Brush.Style:=bsSolid;                                                            //    
  FBgBitMap.Canvas.Brush.Color:=FColor;
  FBgBitMap.Canvas.Pen.Color:=FColor;
  FBgBitMap.Canvas.FillRect(FBgBitMap.Canvas.ClipRect);

  paint_knob_circles(FRCirDown, FRCirUp, FX, FY, FColor1CirDown, FColor2CirDown, FColor1CirUp,      //  
                     FColor2CirUp, FAngleFillCirDown, FAngleFillCirUp, FBgBitMap);

  paint_circle_scale(FRScaleIn, FRScaleOut, FRScaleOutSm, FX, FY, FColorScale, FColorScaleSm,       //  
                      FFont.Size, FIndexMaxEnable, FAngleStartScale, FAngleStopScale, FScaleCode, FBgBitMap, FOscKnobData, FFont.Name);

  if FComboBoxSelect<>nil then                    //    ComboBox
  begin
    FComboBoxSelect.Items.Clear;                  //        
    for i:=0 to FScaleCode.Count-1 do             //     
      FComboBoxSelect.Items.Add(FloatToStr(FOscKnobData[i].dValue)+FOscKnobData[i].strDimension);
    FComboBoxSelect.ItemIndex:=FIndexSelectData;  //   
  end;
end;


//TODO: 2.05 ---------------------------   -------------------------------------
procedure TOscKnob.Paint;
var
  lb_caption: string;
begin
   BitBlt(Canvas.Handle,0,0,Width,Height,FBgBitMap.Canvas.Handle,0,0,SRCCOPY);                      //  

   if FCursorMove then                                                                              //  
     paint_cursor(FRCursorIn, FHCursor, FX, FY, FColorCursorMove, FAngle, FAngleCursor, Canvas)
   else if Enabled then
     paint_cursor(FRCursorIn, FHCursor, FX, FY, FColorCursor, FAngle, FAngleCursor, Canvas)
   else
     paint_cursor(FRCursorIn, FHCursor, FX, FY, FColor, FAngle, FAngleCursor, Canvas);

   Canvas.Brush.Style:=bsClear;
   if FMouseOn or FCursorMove or Focused then   //         
     Canvas.Pen.Color:=FColorSelect             //     
   else                                         //           
     Canvas.Pen.Color:=FColor;                  //    
   Canvas.Ellipse(FX-FRCirDown,FY-FRCirDown,FX+FRCirDown+1,FY+FRCirDown+1);

   lb_caption:=FloatToStr(FOscKnobData[FIndexSelectData].dValue);
   if not FCaptionDimHide then lb_caption:=lb_caption+FOscKnobData[FIndexSelectData].strDimension;  //    
   FCaption.Width:=FCaption.Canvas.TextWidth(lb_caption)+1;                                         //     1 px
   FCaption.Caption:=lb_caption;                                                                    //   

   if FCaptionAvtoCenter then                   //     
   begin
     FCaption.Left:=FX - FCaption.Width div 2 + 1;  // + 1     1 px
     FCaption.Top:= FY - FCaption.Height div 2;
   end;

   if FHintAvtoSet then                         //  
   begin
     Hint:=FOscKnobData[FIndexSelectData].strHint;
     FCaption.Hint:=Hint;
     if FComboBoxSelect<>nil then
       FComboBoxSelect.Hint:=Hint;
   end;

  if FComboBoxSelect<>nil then                  //   ComboBox
    FComboBoxSelect.ItemIndex:=FIndexSelectData;
end;


//TODO: 2.06 ----------------------    -----------------------------------
procedure  TOscKnob.set_exist_angle(angle: double);
var
  index: integer;
begin
  index:=0;                             //     
  if FScaleCode.Count>1 then            //    
  begin
    while (index<FScaleCode.Count-2) and (angle<FOscKnobData[index].dAngle) do              //      
      inc(index);
    if abs(angle-FOscKnobData[index].dAngle)>abs(angle-FOscKnobData[index+1].dAngle) then   //  . 
      inc(index);
  end;

  if (FIndexMaxEnable<>-1) and (index>=FIndexMaxEnable) then                //    
    index:=FIndexMaxEnable-1;                                               //      (FIndexMaxEnable >= 1)

  if (index<>FIndexSelectData) or (angle<>FOscKnobData[index].dAngle) then  //   
  begin
    FIndexSelectData:=index;                    //    
    FAngle:=FOscKnobData[index].dAngle;         //    
    RePaint;                                    //     
    change_index;                               //    
  end;
end;


//TODO: 2.07 ------------------------------   --------------------------------------
procedure TOscKnob.set_cursor_pos(x, y: integer);
var
  dx, dy: integer;
  h, angle: double;
begin
  dx:=FX-x; dy:=FY-y;                   //     
  h:=sqrt(sqr(dx)+sqr(dy));             //      x, y
  if h=0 then exit;                     //      

  if dy<0 then                          //  
   angle:=RadToDeg(ArcSin(-dx/h))+Sign(dx)*180
  else
    angle:=RadToDeg(ArcSin(dx/h));

  if angle>180 then angle:=angle-360;   //    (-180, 180]

  angle:=min(angle,FAngleStartScale);   //   
  angle:=max(angle,FAngleStopScale);    //   

  if (dx=0) and (dy<0) then             //    
  begin
    if FAngleStartScale>=abs(FAngleStopScale) then
      angle:=FAngleStartScale
    else
      angle:=FAngleStopScale;
  end;

  set_exist_angle(angle);               //   
end;


//TODO: 2.08 --------------------     -------------------------------
procedure TOscKnob.change_index;
begin
  if (Assigned(FOnChange)) and not (csDesigning in ComponentState) then FOnChange(Self);
end;


//TODO: 2.09 ----------------     ValueReal ------------------------------
procedure TOscKnob.SetCursorByValueReal(Value: double);
var
  index: integer;
begin
  index:=0;                            //     
  if FScaleCode.Count>1 then           //    
    while (index<FScaleCode.Count-1) and (FOscKnobData[index].dValueReal<>Value) do //        
      inc(index);
  if FOscKnobData[index].dValueReal<>Value then exit;   //      -  

  if index<>FIndexSelectData then               //     
  begin
    FIndexSelectData:=index;                    //    
    FAngle:=FOscKnobData[index].dAngle;         //    
    RePaint;                                    //  
    change_index;                               //    
  end;
end;


//TODO: 2.10 ---------------------      -------------------
procedure TOscKnob.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);                //   
  if (AComponent=FComboBoxSelect) and (Operation=opRemove) then //   ComboBoxSelect
    FComboBoxSelect:= nil;
end;


//TODO: 2.11 ----------------------------    ComboBox ----------------------------
procedure TOscKnob.change_by_combobox(Sender: TObject);
begin
  if Assigned(FComboBoxSelect) then                                         //   ComboBox (    )
  begin
    if FComboBoxSelect.ItemIndex<>FIndexSelectData then                     //   
    begin
      FIndexSelectData:=FComboBoxSelect.ItemIndex;                          //    
      if (FIndexMaxEnable<>-1) and (FIndexSelectData>=FIndexMaxEnable) then //    
       FIndexSelectData:=FIndexMaxEnable-1;                                 //      (FIndexMaxEnable >= 1)
      FAngle:=FOscKnobData[FIndexSelectData].dAngle;                        //    
      RePaint;                                                              //  
      change_index;                                                         //    
    end;
  end;
end;

{***************************************************************************************************
*TODO: 3.0                       TOscKnob                                 *
***************************************************************************************************}

//TODO: 3.00 --------------------      -------------------------
procedure TOscKnob.CMMouseEnter(var msg: TMessage);
begin
  if not (csDesigning in ComponentState) then   //    
  begin
    FMouseOn:=true;                             //   ,    
    RePaint;                                    //  
    inherited;                                  //    
 end;
end;


//TODO: 3.01 ---------------------      --------------------------
procedure TOscKnob.CMMouseLeave(var msg: TMessage);
begin
  if not FCursorMove then                       //     
  begin
    if not (csDesigning in ComponentState) then //    
    begin
      FMouseOn:=false;                          //   ,    
      RePaint;                                  //  
    end;
  end;
  inherited;                                    //    
end;


//TODO: 3.02 --------------------------     ---------------------------------
procedure TOscKnob.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not FCursorMove then SetFocus;             //   
  FCursorMove:=true;                            //  
  set_cursor_pos(x,y);                          //  
  inherited MouseDown(Button,Shift,X,Y);        //    
end;


//TODO: 3.03 ----------------------------    -------------------------------------
procedure TOscKnob.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if FCursorMove then                           //   
    set_cursor_pos(x,y);                        //    
  inherited MouseMove(Shift,X,Y);               //    
end;

//TODO: 3.04 -----------------------     -----------------------------------
procedure TOscKnob.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if FCursorMove then                                   //   
  begin
    FCursorMove:=false;                                 //  
    if (x<0) or (x>=Width) or (y<0) or (y>=Height) then //    
      FMouseOn:=false;                                  //     ,    
    set_cursor_pos(x,y);                                //  
    RePaint;                                            // 
  end;
  inherited MouseUp(Button,Shift,X,Y);                  //    
end;


//TODO: 3.05 --------------------------     ---------------------------------
procedure TOscKnobLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseDown(Button,Shift,X,Y);        //    
  Parent.SetFocus;
end;


//TODO: 3.06 -----------------------------    ------------------------------------
procedure TOscKnob.WMKillFocus(var msg: TWMKILLFOCUS);
begin
  RePaint;                  //  
  inherited;                //    
end;


//TODO: 3.07 -------------------------------    ------------------------------------
procedure TOscKnob.WMSetFocus(var msg: TWMSETFOCUS);
begin
  RePaint;                  //  
  inherited;                //    
end;


//TODO: 3.08 ---------------------       ------------------------------
procedure TOscKnob.KeyDown(var Key: Word; Shift: TShiftState);
var
  index, count: integer;
begin
  index:=FIndexSelectData;                  //   

  case Key of                               //   
    VK_PRIOR,                               // PageUp
    VK_UP,                                  // 
    VK_RIGHT : inc(index);                  // 
    VK_NEXT,                                // PageDown
    VK_LEFT,                                // 
    VK_DOWN  : dec(index);                  // 
    VK_END   : index:=FScaleCode.Count-1;   // End
    VK_HOME  : index:=0;                    // Home
  end;

  if FIndexMaxEnable<>-1 then               //    
    count:=FIndexMaxEnable-1                //       
  else                                      //  
    count:=FScaleCode.Count-1;              //    

  index:=min(index,count);                  //   
  index:=max(index,0);                      //   

  if index<>FIndexSelectData then           //   
  begin
    FIndexSelectData:=index;                //    
    FAngle:=FOscKnobData[index].dAngle;     //    
    RePaint;                                //  
    change_index;                           //    
  end;

  inherited KeyDown(Key, Shift);            //    
end;


//TODO: 3.09 --------------------     ---------------------------------
procedure TOscKnob.WMGetDlgCode(var msg: TWMGetDlgCode);
begin
  inherited;                                //    
  msg.Result := DLGC_WANTARROWS;            //  Win   ,  
end;


//TODO: 3.10 ---------------------     -------------------------------------
procedure TOscKnob.MSHMouseWheel(var msg: TMSHMouseWheel);
var
  index, count: integer;
begin
  index:=FIndexSelectData;                      //   
  index:=index+Round(msg.WheelDelta/7864320);   //   

  if FIndexMaxEnable<>-1 then                   //    
    count:=FIndexMaxEnable-1                    //       
  else                                          //  
    count:=FScaleCode.Count-1;                  //    

  index:=min(index,count);                      //   
  index:=max(index,0);                          //   

  if index<>FIndexSelectData then               //   
  begin
    FIndexSelectData:=index;                    //    
    FAngle:=FOscKnobData[index].dAngle;         //    
    RePaint;                                    //  
    change_index;                               //    
  end;

  inherited;                                    //    
end;


//TODO: 3.11 -----------------------    Enable --------------------------------
procedure TOscKnob.CMEnabledChanged(var msg: TMessage);
begin
  RePaint;                                                              //  
  FCaption.Enabled:=Enabled;                                            //   
  if Assigned(FComboBoxSelect) then FComboBoxSelect.Enabled:=Enabled;   //   
  inherited;                                                            //    
end;


//TODO: 3.12 -----------------------    Visible -------------------------------
procedure TOscKnob.CMVisibleChanged(var msg: TMessage);
begin
  if Assigned(FComboBoxSelect) then FComboBoxSelect.Visible:=Visible;   //   
  inherited;                                                            //    
end;


//TODO: 3.12 -----------------------------    ------------------------------------
procedure TOscKnob.WMSize(var msg: TWMSize);
begin
  FBgBitMap.Width:=Width;
  FBgBitMap.Height:=Height;
  paint_background;
  RePaint;
end;


{***************************************************************************************************
*TODO: 4.0                                                               *
***************************************************************************************************}

//TODO: 4.00 -----       >0 (0 - 12:00,    ),  ----
procedure TOscKnob.SetAngleStartScale(const Value: double);
begin
  FAngleStartScale := Value;
  paint_background;
end;


//TODO: 4.01 -----    <0 (0 - 12:00,    ),  ----
procedure TOscKnob.SetAngleStopScale(const Value: double);
begin
  FAngleStopScale := Value;
  paint_background;
end;


//TODO: 4.02 -----    (0 - 12:00,    ),  ------
procedure TOscKnob.SetAngle(const Value: double);
begin
  FAngle:=Value;                                //  
  FAngle:=min(FAngle,FAngleStartScale);         //   
  FAngle:=max(FAngle,FAngleStopScale);          //   
  set_exist_angle(FAngle);                      //   
end;


//TODO: 4.03 ---------------------------    ,  -----------------------
procedure TOscKnob.SetAngleCursor(const Value: double);
begin
  FAngleCursor := Value;
  RePaint;
end;


//TODO: 4.04 -      (0 - 12:00,    ),  -
procedure TOscKnob.SetAngleFillCirDown(const Value: double);
begin
  FAngleFillCirDown := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.05 -     (0 - 12:00,    ),  -
procedure TOscKnob.SetAngleFillCirUp(const Value: double);
begin
  FAngleFillCirUp := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.06 ---------------------------------   -------------------------------------
procedure TOscKnob.SetColor(const Value: TColor);
begin
  FColor := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.07 ----------------------      -----------------------
procedure TOscKnob.SetColor1CirDown(const Value: TColor);
begin
  FColor1CirDown := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.08 ----------------------      ----------------------
procedure TOscKnob.SetColor1CirUp(const Value: TColor);
begin
  FColor1CirUp := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.09 -----------------------       ----------------------
procedure TOscKnob.SetColor2CirDown(const Value: TColor);
begin
  FColor2CirDown := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.10 ----------------------       ----------------------
procedure TOscKnob.SetColor2CirUp(const Value: TColor);
begin
  FColor2CirUp := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.11 ------------------------------------   -------------------------------------
procedure TOscKnob.SetColorCursor(const Value: TColor);
begin
  FColorCursor := Value;
  RePaint;
end;


//TODO: 4.12 -----------------------      ----------------------------
procedure TOscKnob.SetColorCursorMove(const Value: TColor);
begin
  FColorCursorMove := Value;
  RePaint;
end;


//TODO: 4.13 -----------------------------    -----------------------------------
procedure TOscKnob.SetColorScale(const Value: TColor);
begin
  FColorScale := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.14 -----------------------------    ------------------------------------
procedure TOscKnob.SetColorScaleSm(const Value: TColor);
begin
  FColorScaleSm := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.15 ------------------------    -----------------------------------
procedure TOscKnob.SetColorSelect(const Value: TColor);
begin
  FColorSelect := Value;
  RePaint;
end;


//TODO: 4.16 ----------------------------   ------------------------------------------
procedure TOscKnob.SetFFont(const Value: TFont);
begin
  FFont.Assign(Value);
  paint_background;
  RePaint;
end;


//TODO: 4.17 ------------   : r+h -   ( ) ---------------
procedure TOscKnob.SetHCursor(const Value: integer);
begin
  FHCursor := Value;
  RePaint;
end;


//TODO: 4.18 ----------------------   ()  -----------------------------
procedure TOscKnob.SetRCirDown(const Value: integer);
begin
  FRCirDown := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.19 --------------------   ()  ------------------------------
procedure TOscKnob.SetRCirUp(const Value: integer);
begin
  FRCirUp := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.20 ------------------  ( ) ----------------------------
procedure TOscKnob.SetRCursorIn(const Value: integer);
begin
  FRCursorIn := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.21 -------------------------------   -------------------------------------
procedure TOscKnob.SetRScaleIn(const Value: integer);
begin
  FRScaleIn := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.22 -----------------------     -------------------------------
procedure TOscKnob.SetRScaleOut(const Value: integer);
begin
  FRScaleOut := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.23 -----------------------     --------------------------------
procedure TOscKnob.SetRScaleOutSm(const Value: integer);
begin
  FRScaleOutSm := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.24 ----------------------------    -----------------------------------
procedure TOscKnob.SetScaleCode(const Value: TStrings);
var
  i: integer;
begin
  FScaleCode.Assign(Value);
  paint_background;
  SetAngle(FAngle);     //   
  RePaint;
  
  if FComboBoxSelect<>nil then                      //   ComboBox
  begin
    FComboBoxSelect.Items.Clear;                    //   
    for i:=0 to FScaleCode.Count-1 do               //     
      FComboBoxSelect.Items.Add(FloatToStr(FOscKnobData[i].dValue)+FOscKnobData[i].strDimension);
  end;
end;


//TODO: 4.25 ----------------------------    -----------------------------------
procedure TOscKnobStrings.SetUpdateState(Updating: Boolean);
begin
  OscKnob.paint_background;             //  
  OscKnob.SetAngle(OscKnob.Angle);      //   
end;


//TODO: 4.26 ------------       --------------------------
procedure TOscKnob.SetX(const Value: integer);
begin
  FX := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.27 ------------       --------------------------
procedure TOscKnob.SetY(const Value: integer);
begin
  FY := Value;
  paint_background;
  RePaint;
end;


//TODO: 4.28 -------------------     ---------------------------------
procedure TOscKnob.SetCaptionDimHide(const Value: boolean);
begin
  FCaptionDimHide:=Value;
  RePaint;
end;


//TODO: 4.29 ---------------     -------------------------------
procedure TOscKnob.SetCaptionAvtoCenter(const Value: boolean);
begin
  FCaptionAvtoCenter:=Value;
  RePaint;
end;


//TODO: 4.30 ----------------     ------------------------------
procedure TOscKnob.SetHintAvtoSet(const Value: boolean);
begin
  FHintAvtoSet:=Value;
  RePaint;
end;


//TODO: 4.31 ---------------------------   -------------------------------------
procedure TOscKnob.SetComboBoxSelect(const Value: TComboBox);
var
  i: integer;
begin
  if FComboBoxSelect<>Value then                    //   ComboBox
  begin
    FComboBoxSelect:=Value;                         //    ComboBox
    if Value<>nil then                              //    ComboBox
    begin
      FComboBoxSelect.Style:=csDropDownList;        //         
      FComboBoxSelect.Items.Clear;                  //   
      for i:=0 to FScaleCode.Count-1 do             //     
        FComboBoxSelect.Items.Add(FloatToStr(FOscKnobData[i].dValue)+FOscKnobData[i].strDimension);
      FComboBoxSelect.ItemIndex:=FIndexSelectData;  //   
      FComboBoxSelect.Enabled:=Enabled;             //    
      FComboBoxSelect.Visible:=Visible;             //    
      FComboBoxSelect.OnChange:=change_by_combobox; //  
    end;
  end;
end;


//TODO: 4.32 ------------------------    ------------------------------
procedure TOscKnob.SetIndexMaxEnable(const Value: integer);
begin
  if FIndexMaxEnable<>Value then                                        //   
  begin
    FIndexMaxEnable:=Value;                                             //  
    if (FIndexMaxEnable>=FScaleCode.Count) or (FIndexMaxEnable<=0) then //    
      FIndexMaxEnable:=-1;
    paint_background;                                                   //   
    set_exist_angle(FAngle);                                            //  
    RePaint;                                                            //  ,   
  end;
end;


//TODO: 4.33 ------------------------     ------------------------------
procedure TOscKnob.SetIndexSelectData(const Value: integer);
begin
  if (Value>=0) and (Value<FScaleCode.Count) and (Value<>FIndexSelectData) then //     
  begin
    FIndexSelectData:=Value;                    //    
    FAngle:=FOscKnobData[Value].dAngle;         //    
    RePaint;                                    //  
    change_index;                               //    
  end;
end;


//TODO: 4.34 -----------------------      ---------------------------------
function TOscKnob.GetCurAngle: double;
begin
  GetCurAngle:=FOscKnobData[FIndexSelectData].dAngle;
end;


//TODO: 4.35 -----------------------------   ---------------------------------------
function TOscKnob.GetCurDimension: string;
begin
  GetCurDimension:=FOscKnobData[FIndexSelectData].strDimension;
end;


//TODO: 4.36 ---------------------------    ------------------------------------
function TOscKnob.GetCurExp: double;
begin
  GetCurExp:=FOscKnobData[FIndexSelectData].dExp;
end;


//TODO: 4.37 ----------------------------    --------------------------------------
function TOscKnob.GetCurHint: string;
begin
  GetCurHint:=FOscKnobData[FIndexSelectData].strHint;
end;


//TODO: 4.38 ----------------------      -------------------------------
function TOscKnob.GetCurValue: double;
begin
  GetCurValue:=FOscKnobData[FIndexSelectData].dValue;
end;


//TODO: 4.37 --------------------------    ------------------------------------
function TOscKnob.GetCurValueReal: double;
begin
  GetCurValueReal:=FOscKnobData[FIndexSelectData].dValueReal;
end;

end.
